#Licensed Materials - Property of IBM 
#Rational ClearQuest 
#Copyright IBM Corp. 2008, 2013 All Rights Reserved 

package cqcli::editrecord;

use strict;
use warnings;
require Exporter;

use Error qw(:try);
use cqcli::Exception;
use Inline::Java qw(caught cast) ;
use cqcli::Util;
use cqcli::IOHandler;
use cqcli::commonservice;
use Text::ParseWords;
use Getopt::Long 2.34;


our @ISA = qw(Exporter);

our %EXPORT_TAGS = ( 'all' => [ qw() ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(editrecord);

our $VERSION = '0.01';


#######################################################################
our @ARGVBAACKUP = ();
our %RepetitionCount = ();
our $isLoginOpt = undef;
our $CliSession = undef;
our $JavaReqObj = undef;
our %CmdInfo = ( 'cmdstring' => undef, 'cmdname'  => "edit_record");
our %CmdLogin = ('database'	=>	undef,'schema'=>  undef,
			 'user'		=>	undef,'password'=>  undef);
our $BLANK_MARKER_SET = 0;
our     @fieldOrder = ();
our	%FieldDef = ();
our	%DbIdDef = ();
our	%RIdDef = ();;
our	$TypeDef = "";
our	$DuplicateRid=undef;
our	$ActionDef = undef;

#########################################################################

#########handle getopt warnings begin########################

sub InitParser
{
	my @tmpARGV = @_;
	@ARGVBAACKUP = @ARGV;
	@ARGV = undef;
	$isLoginOpt = undef;
	%RepetitionCount = ();
	%CmdInfo = ('cmdstring' => undef, 'cmdname'  => "edit_record");
	%CmdLogin =('database'	=>undef,'schema'=>  undef,
				'user'	=>	undef,'password'=>  undef);
	@fieldOrder = ();
	%FieldDef = ();
	%DbIdDef = ();
	%RIdDef = ();
	$TypeDef = "";
	$DuplicateRid=undef;
	$ActionDef = undef;
	$CmdInfo{'cmdstring'} =[@tmpARGV];
	($CmdInfo{'cmdname'}  , @ARGV ) = @{$CmdInfo{'cmdstring'}};
	$CliSession = GetSession();
	$JavaReqObj = GetJavaHash();
	return 1;
}

sub CleanUpParser()
{

	@ARGV = @ARGVBAACKUP;
	%RepetitionCount = ();
	%CmdInfo = ('cmdstring' => undef, 'cmdname'  => "edit_record");
	%CmdLogin =('database'	=>undef,'schema'=>  undef,
				'user'	=>	undef,'password'=>  undef);
	@fieldOrder = ();
	%FieldDef = ();
	%DbIdDef = ();
	%RIdDef = ();
	$TypeDef = "";
	$DuplicateRid=undef;
	$ActionDef = undef;
	$CmdInfo{'cmdstring'} = undef;
	$CmdInfo{'cmdname'} =undef;
	$CliSession = undef;
	$JavaReqObj = undef;
}

#########handle getopt warnings end#####################################



sub handle_help_usage
{
	my $opt = $_[0];
	
	if($opt eq "help")	{
		ThrowUsageException($CmdInfo{'cmdname'},$opt);
	}
	else{
		ThrowUsageException($CmdInfo{'cmdname'},$opt);
	}
	return 1;
}

sub handle_ask_mode
{
	my $mode = shift;
	SetCQCLIAskMode(1);
	return 1;
}

sub handle_login_data
{
	my ($opt , $val) = @_;
	my $msg = undef;
	$isLoginOpt = 1;

	if($RepetitionCount{$opt}) {
		ThrowSyntaxErrorException("CQCLI_Repeat_Option","-$opt", $CmdInfo{'cmdname'} );
	}else {
		$RepetitionCount{$opt} = 1;
	}

	if( ($_[0] eq "database") && ($val)	){
		$CmdLogin{'database'} = $val;
	}elsif(($_[0] eq "user") && ($val)) {
		$CmdLogin{'user'} = $val;
	}elsif(($_[0] eq "password") && ($val)) {
		$CmdLogin{'password'} = $val;
	}elsif(($_[0] eq "password") && ($val eq "")){
		$CmdLogin{'password'} = "";
		$BLANK_MARKER_SET = 1 ;
	}
	elsif((($_[0] eq "schema") || ($_[0] eq "master")) && ($val) ) {
		$CmdLogin{'schema'} = $val; 
	}else{
		ThrowSyntaxErrorException("CQCLI_Missing_Value","-$opt",$CmdInfo{'cmdname'});
	}
	return 1;
}

sub handle_action_name
{
	my ($opt , $val) = @_;
	my $msg = undef;

	if($RepetitionCount{'handle_action_name'}) {
		ThrowSyntaxErrorException("CQCLI_Repeat_Action","[$ActionDef and $opt ]", $CmdInfo{'cmdname'} );
	}else {
		$RepetitionCount{'handle_action_name'} = 1;
	}


	if( ($opt eq "action") && ($val)	){
		$ActionDef=$val;
	}elsif(($opt eq "duplicate") && ($val)) {
		$ActionDef="Duplicate";
		$DuplicateRid = $val;
	}else{
		ThrowSyntaxErrorException("CQCLI_Missing_Value","-$opt",$CmdInfo{'cmdname'});
	}
	return 1;

}

sub handle_specific_action_names
{
	my ($opt) = @_;
	my $msg = undef;

	if($RepetitionCount{'handle_specific_action_name'}) {
		ThrowSyntaxErrorException("CQCLI_Repeat_Action","[$ActionDef and $opt ]", $CmdInfo{'cmdname'} );
	}else {
		$RepetitionCount{'handle_specific_action_name'} = 1;
	}
	
	if( ($opt eq "delete") ) {
		$ActionDef="Delete";
	}elsif(($opt eq "unduplicate")) {
		$ActionDef="UnDuplicate";
	}

}

sub handle_recordid_list
{
	my ($opt , $val) = @_;
	my $recordid = undef;
	if( ($opt eq "recid") && ($val)	){
		$RIdDef{$val}=$TypeDef;
	}elsif(($opt eq "dbid") && ($val)) {
		$DbIdDef{$val}=$TypeDef
	}elsif(($opt eq "input") && ($val)){
		if (-e $val){
			open(RIN, "<$val")|| ThrowException('CQCLI_Failure_FileOpen',$val);
			while (<RIN>) {
				$recordid = $_;
				chomp($recordid);
				$recordid = cqclitrim($recordid);
				$RIdDef{$recordid}=$TypeDef;
			}
			close(RIN)|| CLIPrintIdOutput('CQCLI_Info_FileClose',$val);
		}else {
			ThrowException('CQCLI_Failure_FileOpen',$val);
		}

	}else{
		ThrowSyntaxErrorException("CQCLI_Missing_Value","-$opt",$CmdInfo{'cmdname'});
	}
	return 1;

}


sub handle_recordtype_list
{
	my ($opt , $type) = @_;
	my $msg = undef;
	if($RepetitionCount{'$opt'} ) {
		ThrowSyntaxErrorException("CQCLI_Repeat_Option","-$opt", $CmdInfo{'cmdname'} );
	}else {
		$RepetitionCount{'$opt'} = 1;
	}

	if($type){
		$TypeDef = $type;
	}else {
		ThrowSyntaxErrorException("CQCLI_Missing_Value","-$opt", $CmdInfo{'cmdname'} );
	}
	return 1;
}



############################################################################


sub handle_record_list
{
	my ($cnt , @tmpARGV) = @_;
	my $msg = undef;

	if($tmpARGV[$cnt+1])
	{
		if($tmpARGV[$cnt+2])
		{
			$RIdDef{$tmpARGV[$cnt+2]} = $tmpARGV[$cnt+1];
			splice @ARGV,0,3;
		}else {
			ThrowSyntaxErrorException('CQCLI_Missing_RId',$tmpARGV[$cnt]);
		}
	}else {
		ThrowSyntaxErrorException('CQCLI_Missing_RType',$tmpARGV[$cnt]);
	}

	return 1;
}

sub handle_field_list
{
	my ($cnt , @tmpARGV) = @_;
	my $msg = undef;

	if($tmpARGV[$cnt+1])
	{
		if($tmpARGV[$cnt+2])
		{
		
			my $fname = $tmpARGV[$cnt+1];
			$FieldDef{$fname} = $tmpARGV[$cnt+2];
			# Get the size of the hash and that will be the last index where the new key has to be added in the array
			$fieldOrder[(keys(%FieldDef))-1] = $fname;					
			splice @ARGV,0,3;

		}else {
			ThrowSyntaxErrorException('CQCLI_Missing_FValue',$tmpARGV[$cnt+1],$CmdInfo{'cmdname'});
		}
	}else {
		ThrowSyntaxErrorException('CQCLI_Missing_FName',$tmpARGV[$cnt],$CmdInfo{'cmdname'});
	}

	return 1;
}
####################################################################################

sub parseForHelpUsage()
{
	my $ret = undef;

	my $helpRules = {'help|h|?'		=>	\&handle_help_usage	,
					 'man'			=>	\&handle_help_usage	};
	
	Getopt::Long::Configure('autoabbrev','ignore_case','permute','pass_through');
	$ret = GetOptions( %{$helpRules} );
	Getopt::Long::Configure('default');

	return 1;
}

sub parseForInterActiveOption 
{
	my $ret = undef;
	my $askRules = {'ask!'	=>	\&handle_ask_mode	};
	Getopt::Long::Configure('posix_default','ignore_case','permute','pass_through');
	$ret = GetOptions( %{$askRules} );
	Getopt::Long::Configure('default');

	return 1;
}

sub parseForLoginCredentials 
{
	my $ret = undef;
	my $CommonRules = {
		'database|dbs|d:s'					=>  \&handle_login_data			,
		'user|login|u|l:s'					=>	\&handle_login_data			,
		'schema|s|m|master:s'				=>	\&handle_login_data			,
		'password|p:s'						=>	\&handle_login_data			,
		'type:s'							=>	\&handle_recordtype_list			,
	};
	Getopt::Long::Configure( 'autoabbrev','ignore_case','permute','pass_through');
	$ret = GetOptions( %{$CommonRules} );
	Getopt::Long::Configure('default');
	return 1;
}

sub parseForCoreOptions()
{
	my $ret = undef;
	my $CoreOptRules = {
		'action:s'				=>	\&handle_action_name				, 
		'delete'				=>	\&handle_specific_action_names		, 
		'duplicate:s'			=>	\&handle_action_name		        ,
		'unduplicate:s'			=>	\&handle_specific_action_names		,
		'dbid:s'				=>	\&handle_recordid_list				,
		'recid:s'				=>	\&handle_recordid_list				,
		'input:s'				=>  \&handle_recordid_list				,

	};
	Getopt::Long::Configure( 'autoabbrev','ignore_case','permute','pass_through');
	$ret = GetOptions( %{$CoreOptRules} );
	Getopt::Long::Configure('default');

	return 1;
}

sub parseForUnknownOptions
{
	my @tmpARGV = @ARGV;
	my $lmt = $#tmpARGV;
	my $msg = undef;
	for (my $cnt = 0; $cnt <= $lmt;$cnt++)  
	{	
		
		if($tmpARGV[$cnt]) {
			#handle record ids
			if( $tmpARGV[$cnt] =~ /^-(r(ecord)?$|re(cord)?$|rec(ord)?$|reco(rd)?$|recor(d)?$|record$)/i )
			{
				handle_record_list($cnt,@tmpARGV);
				$cnt=$cnt+2;
				next;
			}
			#handle field names ids
			elsif( $tmpARGV[$cnt] =~ /^-(f(ield)?$|fi(eld)?$|fie(ld)?$|fiel(d)?$|field?$)/i )
			{
			
				handle_field_list($cnt,@tmpARGV);
				$cnt=$cnt+2;
				next;
			}elsif($tmpARGV[$cnt] =~ /^(-)(.+)/)
			{
				my $fname = $2;
				my $i = 2;
				my $fieldvaluelist = undef;
				if( exists $tmpARGV[$cnt+1] && !($tmpARGV[$cnt+1] =~ /^-(f(ield)?$|fi(eld)?$|fie(ld)?$|fiel(d)?$|field?$)/i) && !($tmpARGV[$cnt+1] =~ /^(-)(.+)/)){
					$fieldvaluelist = $tmpARGV[$cnt+1];
				} else {	
					ThrowSyntaxErrorException('CQCLI_Missing_Value',$tmpARGV[$cnt],$CmdInfo{'cmdname'});
				}
				while( exists $tmpARGV[$cnt+$i]){
					if(!($tmpARGV[$cnt+$i] =~ /^-(f(ield)?$|fi(eld)?$|fie(ld)?$|fiel(d)?$|field?$)/i) && !($tmpARGV[$cnt+$i] =~ /^(-)(.+)/)){
						$fieldvaluelist = $fieldvaluelist . chr(7) . $tmpARGV[$cnt+$i];
					} else {
						goto NEXT;
					}
					$i++;
				}
				NEXT:$cnt=$cnt+($i-1);
				splice @ARGV,0,$i;
				# Get the size of the hash and that will be the last index where the new key has to be added in the array
				$fieldOrder[keys(%FieldDef)] = $fname;
				$FieldDef{$fname}=$fieldvaluelist;
			}elsif($tmpARGV[$cnt] =~ /^(-+)/){
				ThrowSyntaxErrorException('CQCLI_Invalid_Option',$tmpARGV[$cnt]);
			}else{
				$RIdDef{$tmpARGV[$cnt]} = $TypeDef;
				shift @ARGV;
				next;
			}
		}
		
	}
	return 1;
}

##################################################################################################

sub ValidateSessionInfo()
{
	
	if($isLoginOpt){
		ValidateLoginCredentials();
		if($TypeDef eq "") {
			$TypeDef = getDefaultRecordtype();
		}
		return;
	}elsif($CliSession){
		if($TypeDef eq "") {
			$TypeDef = getDefaultRecordtype();
		}
		return;
	}else{
		ValidateLoginCredentials();
		if($TypeDef eq "") {
			$TypeDef = getDefaultRecordtype();
		}
		return;
	}


}
sub ValidateLoginCredentials()
{
	if(!($CmdLogin{'user'})){
		$CmdLogin{'user'} = GetUserName();
	}
	if(!($CmdLogin{'password'}) && $BLANK_MARKER_SET == 0){
		$CmdLogin{'password'} = GetPassword();
	}
	if(!($CmdLogin{'database'})){
		$CmdLogin{'database'} = GetDBName();
	}
	if(!($CmdLogin{'schema'})){
		$CmdLogin{'schema'} = GetSchemaName();
	}
	$CliSession=CreateSession(%CmdLogin);
	# is not a command scope session
	SetSessionScope(0);
}


sub ValidateActionNames
{
	#If no action set, but there is any field value in arguments, 
	#assume it is an operation of modification and do not prompt for action
	if(!($ActionDef)){
		my $fieldNum=@fieldOrder;
		if($fieldNum>0){
		   $ActionDef="Modify";
		}
	}

	my @recordids =  keys %RIdDef;
	my $idtype = 1;
	if($#recordids <0) {
		@recordids =  keys %DbIdDef;
		$idtype =0;
	}
	if(!($ActionDef)){

		my $actionlist = undef;

		if($#recordids>0){
				my ($id , $type , $idtype) = (undef,$TypeDef,$idtype);
				$id = join(",",@recordids);
				CheckPromptModeValidity();
				my $recordinfo1 = GetRecordInfoObj();
				$actionlist = $recordinfo1->getLegalActionsForMultipleRecords($id,$type,$idtype);
				$actionlist = cqclitrim($actionlist);
				if($actionlist) {
					$ActionDef = AskForActionName($actionlist);
				}else {
					ThrowValidationException('CQCLI_Invalid_NoAction');
				}
		}
		else {
			
			$ActionDef = GetActionName($recordids[0],$TypeDef,$idtype);
		}
	}
	else {
		my $action = $ActionDef;
		my $recordinfo = GetRecordInfoObj();
		my $actionlist = undef;
		
		if($#recordids>0) {
			my $id = join(",",@recordids);
			$actionlist = $recordinfo->getLegalActionsForMultipleRecords($id,$TypeDef,$idtype);
		}
		else {
			$actionlist = $recordinfo->getLegalActions($recordids[0],$TypeDef,$idtype);
		}
		$actionlist = lc(cqclitrim($actionlist));
		$actionlist = $actionlist.",";
		if(!( $actionlist =~/\s*\,?$action\s*\,/i )){
			ThrowValidationException('CQCLI_Failure_Action',$action);
		}
	}
	ValidateDuplicateAction();
	$JavaReqObj->put("action",$ActionDef);	
}

sub ValidateDuplicateAction
{
	# || ($ActionDef =~ /^unduplicate\b/)
	if(($ActionDef =~ /^duplicate\b/i)){
		if(!($DuplicateRid)){
			$DuplicateRid = GetRecordId('1');
		}
		SetDuplicateActionInfo();
	}
	else { return 1; }
}



sub ValidateFieldNames()
{
	my @actualfieldlist = keys %FieldDef;
	my $recordtypeobj = undef;
	my $recordinfoobj = undef;
	my $mflist = undef;
	my @mFlist = ();
	my @resultantmFlist = ();
	my ($key,$fhash)=(undef,undef);
	my %fldhash = ();

	if($#actualfieldlist >= 0) {
		ValidateExistingFields(@actualfieldlist);
	}
	if(IsAskMode())
	{
			my $rid_list = undef;
			my @records =   keys %RIdDef;
			my $id_type = 1;
			
			if($#records < 0) {
				@records =  %DbIdDef;
				$id_type =0;
			}
			
			#Check if this is neccessary
			#if($#records > 0) {
			#	$rid_list = join(",",@records);
			#}else {
			#	$rid_list = $records[0];
			#}
			
			$rid_list = $records[0];
			if(AskForYN('CQCLI_Prompt_AskFieldName')) {
			$fhash = GetAllFieldsInterActively($TypeDef,$rid_list,$id_type);
			%fldhash = ();
			if($fhash){
				%fldhash = %{$fhash};
				foreach $key (keys %{$fhash}) {
					# Get the size of the hash and that will be the last index where the new key has to be added in the array
					$fieldOrder[keys(%FieldDef)] = $key;
					$FieldDef{$key} = $fldhash{$key};					
				}
			}
		}
		SetCQCLIAskMode(undef);
	}

	my ($alist,$mfname,$idtype)=(undef,undef,undef,undef);
	my ($ridlist,$fieldm) = (undef,undef);
	

	@actualfieldlist = keys %FieldDef;
	$recordtypeobj = GetRecordTypeInfoObj($TypeDef);
	$mflist = $recordtypeobj->getMandatoryFieldWithAction($ActionDef);

	if($mflist) {
	@mFlist = split(/\,/,$mflist);
	}

	#for no mandatory fields
	if($#mFlist < 0) {
		goto CONTINUE;
		return 1;
	}
	$alist = join(",",@actualfieldlist);
	$alist = ",".$alist.",";
	foreach  $mfname (@mFlist) {
		if($alist =~ /\,$mfname\,/i) { 
			next;
			}
		else{
			push @resultantmFlist , $mfname;
		}
	}
	
	$recordinfoobj = GetRecordInfoObj();
	my @recordids =   keys %RIdDef;
	$idtype = 1;
	if($#recordids < 0) {
		@recordids =  %DbIdDef;
		$idtype =0;
	}
	
	if($#recordids > 0) {
		$ridlist = join(",",@recordids);
	}else {
		$ridlist = $recordids[0];
	}
	my $rid = undef;
	
	if(!IsInteractiveMode()) {
			goto CONTINUE;
		}

	
	foreach  $fieldm (@resultantmFlist) {

		$rid = $recordinfoobj->isValueRequired($ridlist,$TypeDef,$fieldm,$idtype);
		
		if($rid){
			if($#recordids > 0) {
				CLIPrintIdOutput('CQCLI_Missing_MandatoryValues',$fieldm,$rid,$ridlist);
			}
			$FieldDef{$fieldm} = GetFieldValueInterActively($fieldm,$TypeDef,$rid,$idtype);
			# Get the size of the hash and that will be the last index where the new key has to be added in the array
			$fieldOrder[(keys(%FieldDef))-1] = $fieldm;
		}
	}

	CONTINUE:
	SetFieldNames();
}


sub ValidateExistingFields
{
	my $actualfield = undef;
	my $expectedfield = undef;
	my @recordids =  keys %RIdDef;
	my $idtype = 1;
	if($#recordids <0) {
		@recordids =  keys %DbIdDef;
		$idtype =0;
	}
	my $recordtypeobj = GetRecordTypeInfoObj($TypeDef);

	foreach $actualfield (@fieldOrder) {
		$expectedfield = $recordtypeobj->isValidFieldName($actualfield);
		if(!($expectedfield)) {
			ThrowValidationException('CQCLI_Failure_Field',$actualfield,$TypeDef);
		}elsif($expectedfield =~ /\,/){
			ThrowValidationException('CQCLI_Failure_FieldAmbiguity',$actualfield,$expectedfield,$TypeDef);
		}else{
			if($actualfield ne $expectedfield){
				$FieldDef{$expectedfield}=$FieldDef{$actualfield};
				delete $FieldDef{$actualfield};
				$actualfield = $expectedfield;
			}

			#ValidateFieldValue($actualfield,$FieldDef{$actualfield},$TypeDef,$recordids[0],$idtype);
		}
	}
}

###########################################################################################################

sub ValidateRecordIds()
{
	 
	my ($keys1 , $keys2) = (undef,undef);
	my $id = undef;
	$keys1 = keys %RIdDef;
	$keys2 = keys %DbIdDef;


	if( ($keys1 <=0) && ($keys2 <=0 ) && IsAPIMode()) {
		ThrowValidationException('CQCLI_Failure_MissingIds');
	}
	elsif( ($keys1 <=0) && ($keys2 <=0 ))
	{
		if(!CheckPromptModeValidity('CQCLI_Missing_IdsWarn') && AskForYN('CQCLI_Prompt_AskRid'))
		{	
			do{
				$id = GetRecordId();
				$RIdDef{$id} = $TypeDef;
			} while(AskForYN('CQCLI_Prompt_Continue')) ;
			
		}

		if(!CheckPromptModeValidity('CQCLI_Missing_IdsWarn') && AskForYN('CQCLI_Prompt_AskDbid'))
		{
			do{
				$id = GetDbId();
				$DbIdDef{$id} = $TypeDef;
			} while(AskForYN('CQCLI_Prompt_Continue')) ;
		}
		$keys1 = keys %RIdDef;
		$keys2 = keys %DbIdDef;
		if( ($keys1 <=0) && ($keys2 <=0 )) {
			ThrowValidationException('CQCLI_Failure_MissingIds');
		}
	}
	SetRecordIds();
}

#################################################################################################################



sub SetRecordIds
{
	my ($rectypeinfo,$key,$flag,$ridhash,$dbidhash) = (undef,undef,undef,undef,undef);
	try {
		$rectypeinfo = GetRecordInfoObj();
		if(keys %RIdDef) {
			$ridhash = $CliSession->createOrderedMap();
			foreach $key (keys %RIdDef) {
				if($RIdDef{$key} eq ""){
					$RIdDef{$key} = $TypeDef;
				}
				$ridhash->put($key,$RIdDef{$key});
			}
			$flag = 1;
			$rectypeinfo->checkValidity($ridhash,$flag);
			$JavaReqObj->put("rid",$ridhash);
		}		
		if(keys %DbIdDef){
			$dbidhash = $CliSession->createOrderedMap();
			foreach $key (keys %DbIdDef) {
				if($DbIdDef{$key} eq ""){
					$DbIdDef{$key} = $TypeDef;
				}
				$dbidhash->put($key,$DbIdDef{$key});
			}
			$flag = 0;
			$rectypeinfo->checkValidity($dbidhash,$flag);
			$JavaReqObj->put("dbid",$dbidhash);
		}
	}
	catch cqcli::Exception with {
		my $ex = shift;	
		throw $ex;
	}
	otherwise {
		if (caught("java.lang.Exception")){
			ThrowJavaException($@,"CQCLI_RecordNotFound");
		}
		else{
			ThrowException("CQCLI_Failure_Unexpected",$@);
		}
	}finally {};
}


sub SetFieldNames()
{
	#set request object for successfull field validation
	my $fld = undef;
	my $counter = 0;
	my $key = undef;
	my $tempVar = $CliSession->createOrderedMap();;

	foreach ( @fieldOrder ) {
		if (exists $fieldOrder[$counter]) {
			$key = $fieldOrder[$counter];
			$tempVar->put($key, $FieldDef{$key}) ;	
		
		}
		$counter++;		
	}
	$JavaReqObj->put("field",$tempVar);

}

sub SetDuplicateActionInfo()
{
	try {
		my $rectypeinfo = GetRecordInfoObj();
		my $ridhash = $CliSession->createOrderedMap();
		$ridhash->put($DuplicateRid,$TypeDef);
		my $flag= 1;
		$rectypeinfo->checkValidity($ridhash,$flag);
		$JavaReqObj->put("dup",$DuplicateRid);
	}
	catch cqcli::Exception with {
		my $ex = shift;	
		throw $ex;
	}
	otherwise {
		if (caught("java.lang.Exception")){
			ThrowJavaException($@,"CQCLI_Failure_RId");
		}
		else{
			ThrowException("CQCLI_Failure_Unexpected",$@);
		}
	}finally {};
}

###########################################################################################


sub Parse()
{
	try {
		parseForHelpUsage();
		parseForInterActiveOption();
		parseForLoginCredentials();
		parseForCoreOptions();
		parseForUnknownOptions();
	}
	catch cqcli::Exception with {
		my $ex = shift;		
		throw $ex;
	}
	otherwise {
		if (caught("java.lang.Exception")){
			ThrowJavaException($@,"CQCLI_ErrJava_ParserUnknown");
		}
		else{
			ThrowParseException('CQCLI_Err_ParserUnknown',$@);
		}
		
    }
	finally {
		Getopt::Long::Configure('default');
	};

	return 1;
}

sub Validate()
{
	try {
		ValidateSessionInfo();
		ValidateRecordIds();
		ValidateActionNames();
		ValidateFieldNames();
	}
	catch cqcli::Exception with {
		my $ex = shift;		
		throw $ex;
	}
	otherwise {
		if (caught("java.lang.Exception")){
			ThrowJavaException($@,"CQCLI_ErrJava_ValidationUnknown");
		}
		else{
			ThrowValidationException('CQCLI_Err_ValidationUnknown',$@);
		}		
    }
	finally {	};

	return 1;
}

sub editrecord
{
	my @tmpCmd = @_;
	my $req = undef;
	try {
		InitParser(@tmpCmd);
		Parse();
		Validate();
		$req =  GetJavaPayload("edit_record",$JavaReqObj);
		SetRequestObject($req);
		return 1;
	}finally {
		CleanUpParser();
	};
}













1;
__END__
